/* $Id: ide.h,v 1.1.2.3 1997/02/15 11:13:09 ecd Exp $
 * ide.h: Tadpole SPARCbook 1 specific ide stuff.
 *
 * Copyright(C) 1996  Marc Zyngier   (maz@wild-wind.fr.eu.org)
 * Copyright(C) 1996  Eddie C. Dost  (ecd@skynet.be)
 *
 * Based on the ide driver from 1.2.13pl8
 */

#ifndef _SPARC_IDE_H
#define _SPARC_IDE_H

#include <asm/sbus.h>
#include <asm/openprom.h>
#include <asm/oplib.h>
#include <asm/bitops.h>

#undef MAX_HWIFS
#define MAX_HWIFS		1

#undef SUPPORT_VLB_SYNC
#define SUPPORT_VLB_SYNC	0

struct tadpole_ide_regs {
	volatile unsigned short *data;
	volatile unsigned char *ctl;
	unsigned short base;
};
	
static struct tadpole_ide_regs ide_regs;

static void
tadpole_ide_init(int index, struct linux_sbus_device *dev)
{
	extern ide_hwif_t ide_hwifs[MAX_HWIFS];
	ide_hwif_t *hwif = &ide_hwifs[index];

	prom_apply_sbus_ranges(dev->my_bus, &dev->reg_addrs[0],
			       dev->num_registers, dev);

	ide_regs.data = sparc_alloc_io(dev->reg_addrs[0].phys_addr, 0,
					dev->reg_addrs[0].reg_size, hwif->name,
					dev->reg_addrs[0].which_io, 0x0);
	ide_regs.ctl = sparc_alloc_io(dev->reg_addrs[1].phys_addr, 0,
					dev->reg_addrs[1].reg_size, hwif->name,
					dev->reg_addrs[1].which_io, 0x0);

	hwif->io_base = ide_regs.base = (unsigned long)ide_regs.ctl & 0xfff;
	hwif->irq = dev->irqs[0].pri;
	hwif->present = 1;
}

static void
probe_tadpole_ide(void)
{
	struct linux_sbus *bus;
	struct linux_sbus_device *dev = 0;
	static int index = 0;

	for_each_sbus (bus) {
		for_each_sbusdev (dev, bus) {
			if (!strcmp(dev->prom_name, "id")) {
				tadpole_ide_init(index++, dev);
				if (index > MAX_HWIFS)
					return;
			}
		}
	}
}

#undef IN_BYTE
static __inline__ unsigned char IN_BYTE(int reg)
{
	return ide_regs.ctl[reg - ide_regs.base];
}

#undef OUT_BYTE
static __inline__ void OUT_BYTE(unsigned char val, int reg)
{
	ide_regs.ctl[reg - ide_regs.base] = val;
}

/*
 * Define insw/outsw/insl/outsl here to use the MPI FIFO,
 * then undo the '#if CONFIG_TADPOLE' in ide_input_data()
 * and ide_output_data().
 */


/*
 * Fix the drive ID.
 */
static __inline__ void big_endianize_driveid(struct hd_driveid *id)
{
	unsigned short *p = (unsigned short *)id;
	int i;

	for (i = 0; i < sizeof(*id)/sizeof(*p); i++, p++)
		*p = __swab16(*p);
}

#endif /* _SPARC_IDE_H */
