/* $Id: mpi.h,v 1.1.2.4 1997/03/18 08:00:19 ecd Exp $
 * mpi.h:  Defines specific to the Tadpole SparcBook 1 MBus to PBus Interface.
 *
 * Copyright (C) 1996 Eddie C. Dost (ecd@skynet.be)
 */

#ifndef _SPARC_MPI_H
#define _SPARC_MPI_H 1


#define MPI_HOST_CMD_SER_NO	0x10
#define MPI_HOST_CMD_HW_REV	0x11
#define MPI_HOST_CMD_FW_REV	0x12
#define MPI_HOST_CMD_MAXTEMP	0x13
#define MPI_HOST_CMD_MINTEMP	0x14
#define MPI_HOST_CMD_TEMP	0x15
#define MPI_HOST_CMD_POW_COUNT	0x16
#define MPI_HOST_CMD_POW_SEC	0x17
#define MPI_HOST_CMD_STATUS	0x18
#define MPI_HOST_CMD_ADC	0x19
#define MPI_HOST_CMD_LED_OFF	0x20
#define MPI_HOST_CMD_LED_ON	0x21
#define MPI_HOST_CMD_BEEP	0x22
#define MPI_HOST_CMD_INC_BRIGHT	0x23
#define MPI_HOST_CMD_DEC_BRIGHT	0x24
#define MPI_HOST_CMD_INC_CONT	0x25
#define MPI_HOST_CMD_DEC_CONT	0x26
#define MPI_HOST_CMD_RD_EEPROM	0x27
#define MPI_HOST_CMD_WR_EEPROM	0x28
#define MPI_HOST_CMD_KBD_REP	0x29
#define MPI_HOST_CMD_MS_SCAN	0x2a
#define MPI_HOST_CMD_WR_PORT1	0x2b
#define MPI_HOST_CMD_RD_PORT1	0x2c
#define MPI_HOST_CMD_ACK_INTR	0x30
#define MPI_HOST_CMD_WD_ENABLE	0x40
#define MPI_HOST_CMD_WD_DISABLE	0x41
#define MPI_HOST_CMD_WD_CMD	0x42

#define MPI_MAX_HOST_CMD	(MPI_HOST_CMD_WD_CMD)

#define MPI_RR_RESET		0x10
#define MPI_RR_IORESET		0x08
#define MPI_RR_FIFOSTOP		0x04

#define MPI_CC_MCLK12		0x01
#define MPI_CC_MCLKSTOP		0x02
#define MPI_CC_INPAUSE		0x04
#define MPI_CC_EXTPAUSE		0x08
#define MPI_CC_PAUSECNT_MASK	0x70


struct mpi_regs {
	__volatile__ unsigned char merr_status;	/* Mbus Error Status	RO  */
	unsigned char pad01[15];
	__volatile__ unsigned long ct0_time;	/* Set Count CT0	R/W */
	unsigned char pad02[4];
	__volatile__ unsigned long ct0_value;	/* Current Count CT0	RO  */
	unsigned char pad03[4];
	__volatile__ unsigned char ct0_cntl;	/* Control CT0		R/W */
	unsigned char pad04[7];
	__volatile__ unsigned long ct1_time;	/* Set Count CT1	R/W */
	unsigned char pad05[4];
	__volatile__ unsigned long ct1_value;	/* Current Count CT1	RO  */
	unsigned char pad06[4];
	__volatile__ unsigned char ct1_cntl;	/* Control CT1		R/W */
	unsigned char pad07[23];
	__volatile__ unsigned long int_pend;	/* Interrupt Pending	RO  */
	unsigned char pad08[4];
	__volatile__ unsigned long int_polar;	/* Interrupt Polarity	R/W */
	unsigned char pad09[4];
	__volatile__ unsigned long int_enable;	/* Interrupt Enable	R/W */
	unsigned char pad10[4];
	__volatile__ unsigned long int_ack;	/* Interrupt Acknowledge WO */
	unsigned char pad11[4];
	__volatile__ unsigned char kbd_data;	/* Serial Receive Port	RO  */
	unsigned char pad12[23];
	__volatile__ unsigned char kbd_status;	/* Serial Recv Status	RO  */
	unsigned char pad13[7];
	__volatile__ unsigned char tx_data;	/* Serial Transmit Port	WO  */
	unsigned char pad14[7];
	__volatile__ unsigned char tx_status;	/* Serial Trans Status	RO  */
	unsigned char pad15[7];
	__volatile__ unsigned char reset;	/* Reset		R/W */
	unsigned char pad16[7];
	__volatile__ unsigned char clk_cntl;	/* Clock Control	R/W */
	unsigned char pad17[15];
	__volatile__ unsigned char pm_port;	/* Power Down		WO  */
	unsigned char pad18[7];
	__volatile__ unsigned long fifo_start;	/* FIFO Start Address	R/W */
	unsigned char pad19[4];
	__volatile__ unsigned long fifo_cntl;	/* FIFO Control		R/W */
	unsigned char pad[44];
	__volatile__ unsigned long long *fifo_data;	/* 64 bit only. R/W */
};

extern struct mpi_regs *mpi_regs;

extern void mpi_probe(void);
extern void mpi_init(void);
extern int mpi_host_command(unsigned char cmd,
			    unsigned char *in, int inlen,
			    unsigned char *out, int outlen);

extern __inline__ void mpi_mclkstop(void)
{
	if (!mpi_regs)
		return;
	mpi_regs->clk_cntl |= MPI_CC_MCLKSTOP;
}

extern __inline__ void mpi_poweroff(void)
{
	if (!mpi_regs)
		return;
	mpi_regs->pm_port = 0xff;
}


#endif /* !(_SPARC_MPI_H) */
