/* $Id: psaux.h,v 1.1.2.1 1997/02/15 11:13:12 ecd Exp $
 * psaux.h: Tadpole SPARCbook 1 specific psaux stuff.
 *
 * Copyright(C) 1996  Marc Zyngier   (maz@wild-wind.fr.eu.org)
 * Copyright(C) 1996  Eddie C. Dost  (ecd@skynet.be)
 *
 */

#ifndef _SPARC_PSAUX_H
#define _SPARC_PSAUX_H

#include <asm/sbus.h>
#include <asm/openprom.h>
#include <asm/oplib.h>

#undef QP_IRQ
static int QP_IRQ;

unsigned char aux_device_present=0;

struct tadpole_upc_regs
{
	unsigned char *config_reg_0;
	unsigned char *config_reg_1;
	unsigned char *upc_regs;
};

static struct tadpole_upc_regs upc_regs;
static unsigned char *mouse_regs;

static void
tadpole_upc_init(struct linux_sbus_device *dev)
{
	prom_apply_sbus_ranges(dev->my_bus, &dev->reg_addrs[0],
			       dev->num_registers, dev);

	upc_regs.config_reg_0 = sparc_alloc_io(dev->reg_addrs[0].phys_addr, 0,
				dev->reg_addrs[0].reg_size, "upc_cfg0",
				dev->reg_addrs[0].which_io, 0x0);

	upc_regs.config_reg_1 = sparc_alloc_io(dev->reg_addrs[2].phys_addr, 0,
				dev->reg_addrs[2].reg_size, "upc_cfg1",
				dev->reg_addrs[2].which_io, 0x0);

	upc_regs.upc_regs = sparc_alloc_io(dev->reg_addrs[1].phys_addr, 0,
				dev->reg_addrs[1].reg_size, "upc_regs",
				dev->reg_addrs[1].which_io, 0x0);
}

static void
tadpole_mouse_init(struct linux_sbus_device *dev)
{
	prom_apply_sbus_ranges(dev->my_bus, &dev->reg_addrs[0],
			       dev->num_registers, dev);

	mouse_regs = sparc_alloc_io(dev->reg_addrs[0].phys_addr, 0,
				   dev->reg_addrs[0].reg_size, "psaux",
				   dev->reg_addrs[0].which_io, 0x0);

	QP_IRQ = dev->irqs[0].pri;
}

static void
probe_tadpole_psaux(void)
{
	struct linux_sbus *bus;
	struct linux_sbus_device *dev = 0;

	for_each_sbus (bus) {
		for_each_sbusdev (dev, bus) {
			if (!strcmp(dev->prom_name, "mouse"))
				tadpole_mouse_init(dev);
			if (!strcmp(dev->prom_name, "upc"))
				tadpole_upc_init(dev);
		}
	}
}

static __inline__ unsigned char *get_port_mapping (int intel_port)
{
	switch (intel_port)
	{
	        case 0x310:
	        case 0x311:
		return mouse_regs + intel_port - 0x310;

		case 0x2fa:
		return upc_regs.config_reg_0;

		case 0x3fa:
		return upc_regs.config_reg_1;

		case 0x390:
		case 0x391:
		return upc_regs.upc_regs + intel_port - 0x390;
	}

	printk ("No port mapping for %x in psaux.h !\n", intel_port);
	return NULL;
}


static __inline__ unsigned char IN_BYTE(int reg)
{
	return *(get_port_mapping (reg));
}

#undef  inb
#undef  inb_p
#define inb(reg) IN_BYTE(reg)
#define inb_p(reg) IN_BYTE(reg)


static __inline__ void OUT_BYTE(unsigned char val, int reg)
{
	*(get_port_mapping (reg)) = val;
}

#undef  outb
#undef  outb_p
#define outb(val,reg) OUT_BYTE(val,reg)
#define outb_p(val,reg) OUT_BYTE(val,reg)

#endif /* _SPARC_PSAUX_H */
