/* $Id: tadpole.h,v 1.1.2.4 1997/04/11 22:16:02 davem Exp $
 * tadpole.h:  Defines specific to the Tadpole SPARCbook 1.
 *
 * Copyright (C) 1996  Eddie C. Dost  (ecd@skynet.be)
 */

#ifndef _SPARC_TADPOLE_H
#define _SPARC_TADPOLE_H 1


#include <asm/mpi.h>


#define TADPOLE_ASI_MPI_PHYSICAL 0x21

#define TADPOLE_MPI_INT_ENABLE	0x0d000060
#define TADPOLE_MPI_INT_ACK	0x0d000068
#define TADPOLE_MPI_RXDATA	0x0d000070
#define TADPOLE_MPI_RXSTATUS	0x0d000088
#define TADPOLE_MPI_TXDATA	0x0d000090
#define TADPOLE_MPI_TXSTATUS	0x0d000098
#define TADPOLE_MPI_CLK_CNTL	0x0d0000a8


extern __inline__ void tadpole_mpi_initialize(void)
{
	__asm__ __volatile__ ("sta	%%g0, [%0] %4\n\t" /* mask interrupts */
			      "sub	%%g0, 1, %%o0\n\t"
			      "sta	%%o0, [%1] %4\n\t" /* ack interrupts */
			      "lduba    [%3] %4, %%o0\n\t" /* init CLK_CNTL */
			      "andn	%%o0, %2, %%o0\n\t"
			      "stba	%%o0, [%3] %4\n\t" : :
			      "r" (TADPOLE_MPI_INT_ENABLE),
			      "r" (TADPOLE_MPI_INT_ACK),
			      "i" (MPI_CC_INPAUSE | MPI_CC_EXTPAUSE),
			      "r" (TADPOLE_MPI_CLK_CNTL),
			      "i" (TADPOLE_ASI_MPI_PHYSICAL) : "o0");
}


extern __inline__ void tadpole_mpi_serial_write(unsigned char ch)
{
	int i;

	__asm__ __volatile__ ("lduba	[%0] %1, %%o1\n"
			      "1:\n\t"
			      "and	%%o1, 0x01, %%o1\n\t"
			      "cmp	%%o1, 0x01\n\t"
			      "be,a	1b\n\t"
			      " lduba	[%0] %1, %%o1\n\t" : :
			      "r" (TADPOLE_MPI_TXSTATUS),
			      "i" (TADPOLE_ASI_MPI_PHYSICAL) : "o1", "cc");
	for (i = 0; i < 25 * 200 + 1000; i++)
		/* wait */ ;
	__asm__ __volatile__ ("stba	%0, [%1] %2\n\t" : :
			      "r" (ch), "r" (TADPOLE_MPI_TXDATA),
			      "i" (TADPOLE_ASI_MPI_PHYSICAL));
}

extern __inline__ int tadpole_mpi_serial_read_delay(unsigned char *cptr, int us)
{
	unsigned long enable;
	int ok, status;
	int i = 0;

	us *= 25;
	__asm__ __volatile__ ("lda	[%1] %2, %%o2\n\t"
			      "and	%%o2, %3, %0\n\t"
			      "andn	%%o2, %3, %%o2\n\t"
			      "sta	%%o2, [%1] %2\n\t" :
			      "=r" (enable) :
			      "r" (TADPOLE_MPI_INT_ENABLE),
			      "i" (TADPOLE_ASI_MPI_PHYSICAL),
			      "i" (1 << 11) : "o2");
	
	do {
		__asm__ __volatile__ ("lduba	[%1] %2, %0\n\t" :
				      "=r" (status) :
				      "r" (TADPOLE_MPI_RXSTATUS),
				      "i" (TADPOLE_ASI_MPI_PHYSICAL));
	} while (((status & 0x01) == 0) && (i++ < us));

	if ((ok = (status & 0x01))) {
		__asm__ __volatile__ ("lduba	[%1] %2, %%o2\n\t"
				      "stb	%%o2, [%0]\n\t" : :
				      "r" (cptr), "r" (TADPOLE_MPI_RXDATA),
				      "i" (TADPOLE_ASI_MPI_PHYSICAL) : "o2");
	}

	__asm__ __volatile__ ("lda	[%1] %2, %%o2\n\t"
			      "or	%%o2, %0, %%o2\n\t"
			      "sta	%%o2, [%1] %2\n\t" : :
			      "r" (enable),
			      "r" (TADPOLE_MPI_INT_ENABLE),
			      "i" (TADPOLE_ASI_MPI_PHYSICAL) : "o2");
	return ok;
}


#endif /* !(_SPARC_TADPOLE_H) */
