/* $Id: tadserial.h,v 1.1.2.2 1997/03/28 00:22:47 ecd Exp $
 * tadserial.h: Tadpole SPARCbook 1 specific serial stuff.
 *
 * Copyright(C) 1997  Marc Zyngier   (maz@wild-wind.fr.eu.org)
 * Copyright(C) 1997  Eddie C. Dost  (ecd@skynet.be)
 *
 */

#ifndef _SPARC_TADSERIAL_H
#define _SPARC_TADSERIAL_H

#include <asm/mpi.h>
#include <asm/sbus.h>
#include <asm/openprom.h>
#include <asm/oplib.h>
#include <asm/bitops.h>

struct tadpole_serial_regs
{
	unsigned char *urt_regs;
	unsigned char *modem_regs;
};

static struct tadpole_serial_regs serial_regs = { 0, 0 };

void tadpole_serial_setup(void)
{
	struct linux_sbus *bus;
	struct linux_sbus_device *dev = 0;
        struct linux_prom_registers regs[1];
        struct linux_prom_irqs irqs[1];
	extern struct async_struct rs_table[];
        int node;

	for_each_sbus (bus) {
		for_each_sbusdev (dev, bus) {
			if (!strcmp(dev->prom_name, "urt")) {
				prom_apply_sbus_ranges(dev->my_bus,
					&dev->reg_addrs[0],
					dev->num_registers, dev);
				serial_regs.urt_regs = sparc_alloc_io(
					dev->reg_addrs[0].phys_addr, 0,
					dev->reg_addrs[0].reg_size,
					dev->prom_name,
					dev->reg_addrs[0].which_io, 0x0);
				rs_table[0].irq = dev->irqs[0].pri;
			}
		}
	}

	if (((node = prom_getchild(prom_root_node)) &&
	     (node = prom_searchsiblings(node, "obio")) &&
	     (node = prom_getchild(node)) &&
	     (node = prom_searchsiblings(node, "modem")))) {
		prom_getproperty(node, "reg", (char *)regs, sizeof(regs));
		prom_apply_obio_ranges(regs, 0x1);
		serial_regs.modem_regs = sparc_alloc_io(
			regs[0].phys_addr, 0, regs[0].reg_size,
			"modem", regs[0].which_io, 0x0);
		prom_getproperty(node, "intr", (char *)irqs, sizeof(irqs));
		rs_table[1].irq = irqs[0].pri;
	}

	if (mpi_regs) {
		extern void tadpole_keyboard_init(void);
		tadpole_keyboard_init();
	}
}

static __inline__ unsigned char IN_BYTE(int reg)
{
	switch (reg) {
		case 0x3f8:
		case 0x3f9:
		case 0x3fa:
		case 0x3fb:
		case 0x3fc:
		case 0x3fd:
		case 0x3fe:
		case 0x3ff:
			if (serial_regs.urt_regs)
				return serial_regs.urt_regs[reg - 0x3f8];
			else
				return 0xff;
		case 0x2fe:
		case 0x2f8:
		case 0x2f9:
		case 0x2fa:
		case 0x2fb:
		case 0x2fc:
		case 0x2fd:
		case 0x2ff:
			if (serial_regs.modem_regs)
				return serial_regs.modem_regs[reg - 0x2f8];
			else
				return 0xff;
		default:
			return 0xff;
	}
}

#undef  inb
#undef  inb_p
#define inb(reg)	IN_BYTE(reg)
#define inb_p(reg)	IN_BYTE(reg)


static __inline__ void OUT_BYTE(unsigned char val, int reg)
{
	switch (reg) {
		case 0x3f8:
		case 0x3f9:
		case 0x3fa:
		case 0x3fb:
		case 0x3fc:
		case 0x3fd:
		case 0x3fe:
		case 0x3ff:
			if (serial_regs.urt_regs)
				serial_regs.urt_regs[reg - 0x3f8] = val;
			break;
		case 0x2f8:
		case 0x2f9:
		case 0x2fa:
		case 0x2fb:
		case 0x2fc:
		case 0x2fd:
		case 0x2fe:
		case 0x2ff:
			if (serial_regs.modem_regs)
				serial_regs.modem_regs[reg - 0x2f8] = val;
			break;
		default:
			break;
	}
}

#undef  outb
#undef  outb_p
#define outb(val,reg)	OUT_BYTE(val,reg)
#define outb_p(val,reg)	OUT_BYTE(val,reg)

/*
 * XXX FIXME : Got those numbers from linux/asm-i386/ioctls.h
 *             We do need to stay in sync with it or add it to
 *             linux/asm-sparc/ioctls.h . Anyway, serial.c may
 *             need some hacking to support SunOS ioctls...
 */
#define TIOCMIWAIT      0x545C  /* wait for a change on serial input line(s) */
#define TIOCGICOUNT     0x545D  /* read serial port inline interrupt counts */

void putDebugChar (char dummy)
{
}

int getDebugChar (void)
{
	return 0;
}

#endif /* _SPARC_TADSERIAL_H */
